//--------------------------------------------------------------//
// DX Diffuse Spec
//--------------------------------------------------------------//
//--------------------------------------------------------------//
// DiffuseSpecular_vs_1_1_ps_1_3
//--------------------------------------------------------------//
//--------------------------------------------------------------//
// Pass 0
//--------------------------------------------------------------//

float4x4 matWorldViewProjection;
float4x4 matWorld;
float4x4 matWorldInverseTranspose;
float4x4 matViewInverse;
float4 lightDir;

struct VS_INPUT 
{
    float3 position         : POSITION;
    float3 normal           : NORMAL;
    float4 texCoordDiffuse  : TEXCOORD0;
};

struct VS_OUTPUT 
{
    float4 hPosition        : POSITION;
    float4 texCoordDiffuse  : TEXCOORD0;
    float4 diffAmbColor     : COLOR0;
    float4 specCol          : COLOR1;
};


//------------------------------------
VS_OUTPUT main(VS_INPUT IN) 
{
   VS_OUTPUT OUT;
   OUT.hPosition = mul( float4(IN.position.xyz , 1.0) , matWorldViewProjection);
   OUT.texCoordDiffuse = IN.texCoordDiffuse;

   //calculate our vectors N, E, L, and H
   float3 worldEyePos = matViewInverse[3].xyz;
   
   float3 worldVertPos = mul(IN.position, matWorld).xyz;
   
   float4 N = mul(IN.normal, matWorldInverseTranspose); //normal vector
   
   float3 E = normalize(worldEyePos - worldVertPos); //eye vector
   
   float3 L = normalize( -lightDir.xyz); //light vector
  
   float3 H = normalize(E + L); //half angle vector
   
   //calculate the diffuse and specular contributions
   float  diff = max(0 , dot(N,L));
   
   //Shininess 8.49
   float  spec = pow( max(0 , dot(N,H) ) , 8.49 );
   
   
   
   if( diff <= 0 )
   {
     spec = 0;
   }
   
   float4 materialDiffuse = IN.texCoordDiffuse;
      
   float4 lightAmbient = float4( 0.41, 0.41, 0.41, 1.00 );
   
   float4 lightColor = float4( 0.50, 0.50, 0.50, 0.00 );
   
   float4 materialSpecular = IN.texCoordDiffuse.w;
   
   //output diffuse
   float4 ambColor = materialDiffuse * lightAmbient;
   
   float4 diffColor = materialDiffuse * diff * lightColor ;
   
   OUT.diffAmbColor = diffColor + ambColor;
   
   //output specular
	float4 specColor = materialSpecular * lightColor * spec;
   
   OUT.specCol = specColor;
   
   return OUT;
}
